/*	Renegade Scripts.dll
	Copyright 2011 Tiberian Technologies

	This file is part of the Renegade scripts.dll
	The Renegade scripts.dll is free software; you can redistribute it and/or modify it under
	the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2, or (at your option) any later
	version. See the file COPYING for more details.
	In addition, an exemption is given to allow Run Time Dynamic Linking of this code with any closed source module that does not contain code covered by this licence.
	Only the source code to the module(s) containing the licenced code has to be released.
*/
#pragma once


#define TIMER_CHECK_TARGET 100778801


/*------------------------
Base class for custom AI's
--------------------------*/

class dp88_customAI : public ScriptImpClass
{
public:

	/* -----
	Constructor / Destructor (cleanup debug file if applicable
	----- */

	dp88_customAI() { debugFile = NULL; };
	~dp88_customAI() { if ( debugFile != NULL ) { fclose(debugFile); } };




	/* -----
	Variables
	----- */

	// Priority and weapon choice for each target type
	float priority_infantry;			bool primary_infantry;
	float priority_lightVehicle;		bool primary_lightVehicle;
	float priority_heavyVehicle;		bool primary_heavyVehicle;
	float priority_VTOL;				bool primary_VTOL;
	float priority_building;			bool primary_building;

	// Priority modifiers
	float modifier_distance, modifier_target_damage, modifier_target_value;

	// Attack ranges
	int primary_minRange, primary_maxRange;
	int secondary_minRange, secondary_maxRange;

	// Current target state
	int targetID;
	int targetLastSeen;
	float targetPriority;
	bool primary_target;

	// Debug state
	bool debug;
	FILE* debugFile;




	/* -----
	Events
	----- */

	virtual void Created( GameObject *obj );
	virtual void Action_Complete( GameObject *obj, int action_id, ActionCompleteReason complete_reason  );
	//virtual void Enemy_Seen ( GameObject *obj, GameObject *enemy );
	//virtual void Timer_Expired( GameObject *obj, int number );


	/* ----
	Functions
	----- */
	
	virtual void Init( GameObject *obj );
	virtual void loadSettings( GameObject *obj, bool loadSecondaryFireSettings = true, bool loadBuildingTargetSettings = false, bool oldSettingNames = false );
	
	float getDistance ( GameObject *obj1, GameObject *obj2 );
	virtual float getPriority( GameObject *obj, GameObject *target );
	virtual float getPriority( GameObject *obj, int target_id );
	bool getPrimary ( GameObject *target );

	/* Utility functions for both priority calculations and AI scripts to utilise */
	virtual bool IsVehicleEmpty( VehicleGameObj* vobj );
	virtual bool IsVehicleAIEnabled( VehicleGameObj* vobj );
};


/*------------------------
Offensive Tank AI
--------------------------*/

class dp88_tankAI_Offensive : public dp88_customAI
{
	/* ----
	Variables
	----- */

	// Preferred attack ranges
	int primary_prefRange, secondary_prefRange;
	bool movingToTarget;

	// Other
	int retreatDamageAmount;
	FILE *debugFile;
	
	// Events
	void Created( GameObject *obj );
	void Enemy_Seen ( GameObject *obj, GameObject *enemy );
	void Timer_Expired( GameObject *obj, int number );
};



/*------------------------
Turret AI
	Base class for turret type AI scripts
--------------------------*/

class dp88_AI_Turret : public dp88_customAI
{
public:
	// Events
	virtual void Created( GameObject *obj );
	virtual void Enemy_Seen ( GameObject *obj, GameObject *enemy );
	virtual void Timer_Expired( GameObject *obj, int number );

protected:
	bool requiresPower, splashInfantry;

	virtual void Init( GameObject *obj );
	virtual void loadSettings( GameObject *obj, bool loadSecondaryFireSettings = true, bool loadBuildingTargetSettings = false );

	// These can be overloaded from their default functionality as required, allowing the default
	// enemy seen procedure to be reused even when the behaviour of these checks has been altered
	virtual bool checkTeam ( GameObject* obj, GameObject* target );
	virtual bool checkRange ( GameObject* obj, GameObject* target, bool primary );
	virtual bool checkPowerState( GameObject* obj );




	/* These functions are used to initiate and control the AI actions and can be overloaded from
	their defaults if required to provide custom functionality, such as delaying an attack whilst
	waiting for a chargeup / popup animation to occur. */

	// These is called when a valid target has been identified and selected as the highest priority
	// target, the turret should begin attacking in this function. Note that this may be called
	// while another attack is already in progress, either normal or splash.
	virtual void attackTarget ( GameObject* obj, GameObject* target, bool primary );

	// This is called instead of attackTarget when we are set to splash infantry instead of shooting
	// at them directly. Whilst we are attacking an infantry unit with splash this will be called
	// regularly on a timer to update the location of the target. Note that this may be called
	// while another attack is already in progress, either normal or splash.
	virtual void attackLocation ( GameObject* obj, Vector3 location, bool primary );

	// This is called when the target is no longer valid and the turret should stop attacking, this
	// is called to stop both attackTarget and splashLocation attacks.
	virtual void stopAttacking ( GameObject* obj );
};



/*------------------------
Heavy Vehicle Marker
  Dummy script to mark a vehicle as being heavy
--------------------------*/

class dp88_AI_heavyVehicleMarker : public ScriptImpClass
{};