/*	Renegade Scripts.dll
	Copyright 2011 Tiberian Technologies

	This file is part of the Renegade scripts.dll
	The Renegade scripts.dll is free software; you can redistribute it and/or modify it under
	the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2, or (at your option) any later
	version. See the file COPYING for more details.
	In addition, an exemption is given to allow Run Time Dynamic Linking of this code with any closed source module that does not contain code covered by this licence.
	Only the source code to the module(s) containing the licenced code has to be released.
*/
#pragma once

#include "scripts.h"
#include "engine.h"



/*------------------------
Random weather script
--------------------------*/

class dp88_randomWeather : public ScriptImpClass
{
	void Created ( GameObject* obj );
	void Timer_Expired ( GameObject* obj, int number );
};





/*------------------------
Indirect targetting script
--------------------------*/

class dp88_indirectTargetting : public JFW_Key_Hook_Base
{
	void Created ( GameObject* obj );
	void Destroyed ( GameObject* obj );
	void Custom ( GameObject* obj, int type, int param, GameObject* sender );
	void KeyHook();

	/* General variables. ID of this object, ID of the pilot, current
	targetting level. */
	int objectID, pilotID, * targettingLevel;
};


class dp88_indirectTargetting_Dec : public JFW_Key_Hook_Base
{
	void Created ( GameObject* obj );
	void Custom ( GameObject* obj, int type, int param, GameObject* sender );
	void KeyHook();

	/* General variables. ID of this object, ID of the pilot, current
	targetting level. */
	int objectID, pilotID, * targettingLevel;
};





/*------------------------
Unit sounds script
--------------------------*/

class dp88_unitSounds : public ScriptImpClass
{
	void Created ( GameObject* obj );
	void Damaged( GameObject *obj, GameObject *damager, float amount );
	void Killed ( GameObject *obj, GameObject *killer );
	void Timer_Expired ( GameObject* obj, int number );
	void Custom ( GameObject* obj, int type, int param, GameObject* sender );

	// Enabled states
	bool damagedEnabled;
	bool heavilyDamagedEnabled;
	bool healedEnabled;
	bool fallingDamageEnabled;

	// Counters
	int healAmount;
};



/*------------------------
Unique Infantry Controller
--------------------------*/

class dp88_uniqueInfantry_controller : public JFW_Object_Created_Hook_Base
{
	void Created ( GameObject* obj );
	void ObjectCreateHook ( GameObject* obj );
	void Destroyed ( GameObject* obj );

	char* infantry_preset;
	int team_id;
	int max_count;
	int percent_chance;
};



/*------------------------
Damage animation script
--------------------------*/

class dp88_damageAnimation : public ScriptImpClass
{
	void Created ( GameObject* obj );
	void Damaged( GameObject *obj, GameObject *damager, float amount );
	void Timer_Expired ( GameObject* obj, int number );
	void Animation_Complete ( GameObject *obj, const char *animation_name );

	void SetDamageAnimation ( GameObject* obj );

	int currentDamageLevel;
	int damageLevelBoundaries[6];
	int damageLevelStartFrames[6];
	int damageLevelEndFrames[6];
	int damageLevelLowPowerStartFrames[6];
	int damageLevelLowPowerEndFrames[6];
	bool loopAnimation;
	bool basePowerState;
};



/*------------------------
Object Annoucements script
--------------------------*/

class dp88_objectAnnouncements : public ScriptImpClass
{
	void Created ( GameObject* obj );
	void Damaged ( GameObject *obj, GameObject *damager, float amount );
	void Killed ( GameObject *obj, GameObject *killer );

	time_t lastAnnouncementTime;
	int minimumAnnounceInterval;

	unsigned long team0_underAttackStringId;
	unsigned long team1_underAttackStringId;
	unsigned long team0_destroyedStringId;
	unsigned long team1_destroyedStringId;

	unsigned long team0_underAttackSoundId;
	unsigned long team1_underAttackSoundId;
	unsigned long team0_destroyedSoundId;
	unsigned long team1_destroyedSoundId;
};





/*------------------------
Chrono Tank Script
--------------------------*/

class dp88_chronoTank : public JFW_Key_Hook_Base
{
	void Created ( GameObject* obj );
	void Killed ( GameObject *obj, GameObject *killer );
	void Custom( GameObject *obj, int type, int param, GameObject *sender );
	void Timer_Expired ( GameObject* obj, int number );
	void KeyHook();

	void Shift_Out_Begin(GameObject* obj);
	void Shift_Out_Complete(GameObject* obj);
	void Shift_Out_Cleanup(GameObject* obj);
	void Shift_In_Begin(GameObject* obj);
	void Shift_In_Complete(GameObject* obj);
	void Shift_In_Cleanup(GameObject* obj);

	int state;
	int object_id;		// For KeyHook (obj is not provided)
	int driver_id;
	int shifted_vehicle_object_id;

private:
	// Define states
	enum CT_STATES
	{
		CT_STATE_CHARGING,
		CT_STATE_CHARGED,
		CT_STATE_SHIFTING_OUT,
		CT_STATE_SHIFTED_OUT_PENDING_CLEANUP_1,
		CT_STATE_SHIFTED_OUT_PENDING_CLEANUP_2,
		CT_STATE_SHIFTED_OUT,
		CT_STATE_SHIFTING_IN,
		CT_STATE_SHIFTED_IN_PENDING_CLEANUP_1,
		CT_STATE_SHIFTED_IN_PENDING_CLEANUP_2
	};
};



/*------------------------
Spawn Vehicle Part script
--------------------------*/

class dp88_spawnVehiclePart : public ScriptImpClass
{
	void Created ( GameObject* obj );
	void Timer_Expired( GameObject *obj, int number );
	void Killed ( GameObject* obj, GameObject *killer );
	void Destroyed( GameObject* obj );

	int partObjectId;
};



/*------------------------
Custom Points script
--------------------------*/

class dp88_customPoints : public ScriptImpClass
{
	void Created ( GameObject* obj );
	void Damaged ( GameObject *obj, GameObject *damager, float amount );
	void Killed ( GameObject* obj, GameObject *killer );

	float killPoints;
	float damagePoints;
	float repairPoints;
	float killMoney;
	float damageMoney;
	float repairMoney;
};